#!/usr/bin/env python2

import dns.resolver
import fcntl
import netaddr
import netifaces
import re
import socket
import struct
import sys

# interfaces which should not be used
ignore = '^(lo|virbr|docker|vnet|fuelbr)'

def is_interface_up(interface):
    addr = netifaces.ifaddresses(interface)
    return netifaces.AF_INET in addr

# http://stackoverflow.com/questions/24196932/how-can-i-get-the-ip-address-of-eth0-in-python
def get_address(interface):
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(
        s.fileno(),
        0x8915,
        struct.pack('256s', interface[:15])
    )[20:24])

# http://stackoverflow.com/questions/936444/retrieving-network-mask-in-python
def get_netmask(interface):
    return socket.inet_ntoa(fcntl.ioctl(
        socket.socket(
            socket.AF_INET,
            socket.SOCK_DGRAM),
        35099,
        struct.pack('256s', interface)
    )[20:24])

# http://stackoverflow.com/questions/2761829/python-get-default-gateway-for-a-local-interface-ip-address-in-linux/2761952
def get_gateway():
    """Read the default gateway directly from /proc."""
    with open("/proc/net/route") as fh:
        for line in fh:
            fields = line.strip().split()
            if fields[1] != '00000000' or not int(fields[3], 16) & 2:
                continue
            return socket.inet_ntoa(struct.pack("<L", int(fields[2], 16)))

# https://github.com/donjajo/py-world/blob/master/resolvconfReader.py
def get_resolvers():
    resolvers = []
    try:
        with open( '/etc/resolv.conf', 'r' ) as resolvconf:
            for line in resolvconf.readlines():
                if 'nameserver' in line:
                    resolvers.append(line.split()[1])
        return resolvers
    except IOError as error:
        return error.strerror

def get_search_domains():
    try:
        with open( '/etc/resolv.conf', 'r' ) as resolvconf:
            for line in resolvconf.readlines():
                if 'search' in line:
                    return line.split()[1:]
            return ''
    except IOError as error:
        return error.strerror

# check interfaces list and pick first configured one
for i in netifaces.interfaces():
    # skip if interface should be ignored
    if re.match(ignore, i):
        continue
    # check if interface is up and stop when it is
    if is_interface_up(i):
        interface = i
        break

# exit if none of interfaces is configured
try:
    interface
except:
    sys.exit()

address = get_address(interface)
netmask = get_netmask(interface)

resolvers = get_resolvers()
resolver = dns.resolver.Resolver(configure=False)
resolver.nameservers = resolvers
address_remote = str(resolver.query(socket.getfqdn(), 'a')[0])

if netaddr.IPAddress(address_remote) in netaddr.IPNetwork('%s/%s' % (address, netmask)):
    print 'autonetwork_interface=%s' % interface
    print 'autonetwork_address=%s' % address_remote
    print 'autonetwork_netmask=%s' % netmask
    print 'autonetwork_gateway=%s' % get_gateway()
    print 'autonetwork_nameservers=%s' % ' '.join(resolvers)
    # only apply search domains if available
    search_domains = get_search_domains()
    if search_domains:
        print 'autonetwork_search=%s' % ' '.join(search_domains)
